from rocolib.api.components.Component import Component

c = Component()

#Parameters for the sheath that would cover the wheels and the ESP32 Stack
#Parameters that will change the dimension of the car
c.addParameter("length", 76, paramType="length", minValue=65) #tested minValue
c.addParameter("height", 36, paramType="length", minValue=36) #tested minValue
c.addParameter("width", 60, paramType="length", minValue=60) #tested minValue

#This car will use the customized ESP32 stack in dimensions.py and the FS90R servo
#Change here if you have another board
c.addParameter("driveservo", "fs90r", paramType="dimension")
c.addParameter("brains", "esp32stack", paramType="dimension")

#Cover/body of the car (I needed one that folds -90; will try on SimpleRectBeam)
for i in range(1, 3):
    c.addSubcomponent("rect%d" %i, "Rectangle")
    c.addConstraint(("rect%d" %i, "l"), "length")
    c.addConstraint(("rect%d" %i, "w"), "height")

for i in range(3, 5):
    c.addSubcomponent("rect%d" %i, "Rectangle")
    c.addConstraint(("rect%d" %i, "l"), "length")
    c.addConstraint(("rect%d" %i, "w"), "width")

#TODO: ADD TABS FOR ATTACHMENT
# c.addConnection(("rect2", "b"), ("rect3", "t"), angle=90)
c.addConnection(("rect3", "b"), ("rect1", "b"), angle=-90)
c.addConnection(("rect2", "b"), ("rect4", "b"), angle=-90)
c.addConnection(("rect4", "t"), ("rect1", "t"), angle=-90)

#Attaching servos to the car
c.addSubcomponent("rect0", "Rectangle")
c.addConstraint(("rect0", "l"), ("height", "driveservo"), "x[0]-getDim(x[1], 'motorwidth')")
c.addConstraint(("rect0", "w"), "driveservo", "getDim(x, 'motorwidth')")

c.addSubcomponent("sheathsplit", "SplitEdge")
c.addConstraint(("sheathsplit", "toplength"), "length", "(x,)")
c.addConstraint(("sheathsplit", "botlength"), ("length", "driveservo"), "(x[0] * 0.0691, getDim(x[1], 'motorwidth'), "
                                                                        "(x[0]-(getDim(x[1], 'motorwidth') + x[0] * 0.0691)))")

c.addConnection(("sheathsplit", "topedge0"), ("rect2", "t"))
c.addConnection(("sheathsplit", "botedge1"), ("rect0", "r"), angle=180)

for i in range(2):
    c.addSubcomponent("rTriangle%d" %i, "RightTriangle")
    c.addConstraint(("rTriangle%d" %i, "base"), "driveservo", "getDim(x, 'motorwidth')")
    c.addConstraint(("rTriangle%d" %i, "height"), "driveservo", "getDim(x, 'motorwidth')")

c.addConnection(("rect0", "l"), ("rTriangle0", "bedge"))
c.addConnection(("rTriangle0", "hedge"), ("rTriangle1", "hedge"), angle=180)

#Four wheels using Wheel.yaml
c.addSubcomponent("fleft", "Wheel", invert=True, inherit="angle")
c.addSubcomponent("bright", "Wheel", invert=True, inherit="angle")

#Setting the parameters for the Wheels
for servo in ("bright", "fleft"):
    c.addConstraint((servo, "servo"), "driveservo")
    c.addConstraint((servo, "length"), ("length", "brains"), '((x[0] - getDim(x[1], "width")))')
    c.addConstConstraint((servo, "center"), False)
    c.addConstraint((servo, "radius"), "brains", '(getDim(x, "height") / 1.35)')

#Support for the middle of the front and back wheels so that they won't collapse or wiggle
for i in range(1, 3):
    c.addSubcomponent("between%d" %i, "SimpleRectBeam")
    c.addConstraint(("between%d" %i, "length"), ("width", "driveservo"), "x[0]- 2 * getDim(x[1], 'motorheight')")
    c.addConstraint(("between%d" % i, "depth"), "driveservo", "getDim(x, 'motorwidth')")
    c.addConstraint(("between%d" % i, "width"), "driveservo", "getDim(x, 'motorwidth')")

#Back between support
c.addConnection(("fleft", "topedge2"), ("between1", "topedge1"), angle=90)
c.addConnection(("between1", "botedge1"), ("bright", "botedge2"), angle=90)
c.addConnection(("between2", "topedge1"), ("bright", "topedge2"), angle=90)
c.addConnection(("between2", "botedge1"), ("fleft", "botedge2"), angle=90, tabWidth=10)

# Flipping right wheels to match the orientation of left wheels
c.addConstConstraint(("bright", "flip"), True)

c.addConnection(("rTriangle1", "bedge"), ("fleft", "botedge0"))

c.inheritAllInterfaces("fleft")
c.inheritAllInterfaces("bright")
c.inheritAllInterfaces("rect1")
c.inheritAllInterfaces("rect2")
c.inheritAllInterfaces("rect3")
c.inheritAllInterfaces("rect4")

c.toLibrary("HalfCar")